/*
 * stilltest.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/player.h>

extern "C" {
#include "ffmpeg/avcodec.h"
};

static const char *VERSION        = "0.0.1";
static const char *DESCRIPTION    = "Enter description for 'stilltest' plugin";
static const char *MAINMENUENTRY  = "Stilltest";

class cMyPlayer : public cPlayer
{
public:
	void StillImage(unsigned char * data, int length);
};

void cMyPlayer::StillImage(unsigned char * data, int length)
{
	DeviceStillPicture(data, length);
}

class cMyControl : public cControl
{
private:
	cMyPlayer * player;

	AVCodec * codec;
	AVCodecContext * context;	

	void DrawStillImage(int color);
public:
	cMyControl(void);
	virtual ~cMyControl();
	virtual void Hide(void);
	virtual eOSState ProcessKey(eKeys Key);
};

cMyControl::cMyControl(void)
:	cControl(player = new cMyPlayer)
{
	// find the mpeg1 video encoder
	codec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
	if (codec == NULL)
	{
		printf("codec not found\n");
		return;
	}
	context = avcodec_alloc_context();

	context->bit_rate = 1000000;
	context->width = 720;  
	context->height = 576;
	context->frame_rate = 25;  
	context->frame_rate_base = 1;
	context->gop_size = 0; // intra only
	context->max_b_frames = 0;
	context->flags |= CODEC_FLAG_QSCALE;

	// open it
	if (avcodec_open(context, codec) < 0)
	{
		printf("could not open codec\n");
		return;
	}
}

cMyControl::~cMyControl()
{
	delete player;
	player = NULL;
}

void cMyControl::Hide(void)
{
}

eOSState cMyControl::ProcessKey(eKeys Key)
{
	switch (Key)
	{
		case kRed:
			DrawStillImage(0);
			break;
		case kGreen:
			DrawStillImage(1);
			break;
		case kYellow:
			DrawStillImage(2);
			break;
		case kBlue:
			DrawStillImage(3);
			break;
		case kBack:
			return osEnd;
		default:
			break;
	}
	return osContinue;
}

void cMyControl::DrawStillImage(int color)
{
	unsigned char * inputBuffer, * pictureBuffer, * outputBuffer;
	AVFrame * input, * picture;
	int size, outSize;
	int x, y;
	unsigned int * inputPtr;

	inputBuffer = new unsigned char[context->width * context->height * 4];
	if (inputBuffer == NULL)
	{
		return;
	}
	inputPtr = (unsigned int *) inputBuffer;
	for (y = 0; y < context->height; y++)
	{
		for (x = 0; x < context->width; x++)
		{
			if (color == 0)
				*inputPtr = 0xFF << 16 | 0x00 << 8 | 0x00;
			else if (color == 1)
				*inputPtr = 0x00 << 16 | 0xFF << 8 | 0x00;
			else if (color == 2)
				*inputPtr = 0xFF << 16 | 0xFF << 8 | 0x00;
			else if (color == 3)
				*inputPtr = 0x00 << 16 | 0x00 << 8 | 0xFF;
			inputPtr++;
		}
	}

	input = avcodec_alloc_frame();
	picture = avcodec_alloc_frame();
	avpicture_fill((AVPicture *) input, inputBuffer, PIX_FMT_RGBA32, context->width, context->height);

	size = context->width * context->height;
	pictureBuffer = new unsigned char[(size * 3) / 2]; // size for YUV 420
	picture->data[0] = pictureBuffer;
	picture->data[1] = picture->data[0] + size;
	picture->data[2] = picture->data[1] + size / 4;
	picture->linesize[0] = context->width;
	picture->linesize[1] = context->width / 2;
	picture->linesize[2] = context->width / 2;

	img_convert((AVPicture *) picture, PIX_FMT_YUV420P,
		(AVPicture *) input, PIX_FMT_RGBA32,
		context->width, context->height);

	picture->quality = 1; // between 1 and 31

	outputBuffer = new unsigned char[500000];

	// encode the image
	outSize = avcodec_encode_video(context, outputBuffer, sizeof(outputBuffer), picture);
	printf("%d\n", outSize);

	// display the i-frame
	player->StillImage(outputBuffer, outSize);

	delete[] inputBuffer;
	delete[] pictureBuffer;
	delete[] outputBuffer;

	free(input);
	free(picture);
}

class cPluginStilltest : public cPlugin
{
private:
	// Add any member variables or functions you may need here.
public:
	cPluginStilltest(void);
	virtual ~cPluginStilltest();
	virtual const char *Version(void) { return VERSION; }
	virtual const char *Description(void) { return DESCRIPTION; }
	virtual const char *CommandLineHelp(void);
	virtual bool ProcessArgs(int argc, char *argv[]);
	virtual bool Initialize(void);
	virtual bool Start(void);
	virtual void Housekeeping(void);
	virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
	virtual cOsdObject *MainMenuAction(void);
	virtual cMenuSetupPage *SetupMenu(void);
	virtual bool SetupParse(const char *Name, const char *Value);
};

cPluginStilltest::cPluginStilltest(void)
{
	// Initialize any member variables here.
	// DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
	// VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginStilltest::~cPluginStilltest()
{
	// Clean up after yourself!
}

const char *cPluginStilltest::CommandLineHelp(void)
{
	// Return a string that describes all known command line options.
	return NULL;
}

bool cPluginStilltest::ProcessArgs(int argc, char *argv[])
{
	// Implement command line argument processing here if applicable.
	return true;
}

bool cPluginStilltest::Initialize(void)
{
	// Initialize any background activities the plugin shall perform.

	// must be called before using avcodec lib
	avcodec_init();
	// register all the codecs (you can also register only the codec
   	// you wish to have smaller code)
	avcodec_register_all();

	return true;
}

bool cPluginStilltest::Start(void)
{
	// Start any background activities the plugin shall perform.
	return true;
}

void cPluginStilltest::Housekeeping(void)
{
	// Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginStilltest::MainMenuAction(void)
{
	// Perform the action when selected from the main VDR menu.
	cControl::Launch(new cMyControl);
	return NULL;
}

cMenuSetupPage *cPluginStilltest::SetupMenu(void)
{
	// Return a setup menu in case the plugin supports one.
	return NULL;
}

bool cPluginStilltest::SetupParse(const char *Name, const char *Value)
{
	// Parse your own setup parameters and store their values.
	return false;
}

VDRPLUGINCREATOR(cPluginStilltest); // Don't touch this!

